-- X-Perl UnitFrames
-- Author: Zek <Boodhoof-EU>
-- License: GNU GPL v3, 29 June 2007 (see LICENSE.txt)

local XPerl_Player_Events = {}
local isOutOfControl = nil
local playerClass, playerName
local conf, pconf
XPerl_RequestConfig(function(new) conf = new pconf = conf.player if (XPerl_Player) then XPerl_Player.conf = conf.player end end, "$Revision: 652 $")
local perc1F = "%.1f"..PERCENT_SYMBOL
local percD = "%d"..PERCENT_SYMBOL

--[===[@debug@
local function d(...)
	ChatFrame1:AddMessage(format(...))
end
--@end-debug@]===]

local format = format
local GetNumGroupMembers = GetNumGroupMembers
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitName = UnitName
local UnitPowerType = UnitPowerType
local XPerl_Unit_Initialize = XPerl_Unit_Initialize

local Xperl_Player_InitClassResource
local XPerl_Player_InitDK

local XPerl_PlayerStatus_OnUpdate
local XPerl_Player_HighlightCallback

----------------------
-- Loading Function --
----------------------
function XPerl_Player_OnLoad(self)
	XPerl_SetChildMembers(self)
	XPerl_BlizzFrameDisable(PlayerFrame)
	self.unit = "player"

	CombatFeedback_Initialize(self, self.hitIndicator.text, 30)

	local events = {"UPDATE_FACTION", "PLAYER_CONTROL_LOST", "PLAYER_CONTROL_GAINED", 
					"PLAYER_TALENT_UPDATE", "UPDATE_SHAPESHIFT_FORM", "RUNE_TYPE_UPDATE", "RUNE_POWER_UPDATE", 
					"UNIT_AURA", "UNIT_COMBAT", "RAID_TARGET_UPDATE", 
					"UNIT_LEVEL", "UNIT_FACTION", "UNIT_FLAGS", "PLAYER_ENTERING_WORLD", "PLAYER_REGEN_DISABLED", 
					"PLAYER_REGEN_ENABLED", "PLAYER_UPDATE_RESTING", "UPDATE_EXHAUSTION", "PARTY_LEADER_CHANGED", 
					"PARTY_LOOT_METHOD_CHANGED", "GROUP_ROSTER_UPDATE", "UNIT_ENTERED_VEHICLE", 
					"UNIT_EXITED_VEHICLE", "PLAYER_ROLES_ASSIGNED"}
	for i,event in pairs(events) do
		if (strsub(event, 1, 5) == "UNIT_") then
			self:RegisterUnitEvent(event, "player", "vehicle")
		else
			self:RegisterEvent(event)
		end
	end	
	self:RegisterUnitEvent("PLAYER_FLAGS_CHANGED", "player", "vehicle")
	
	self:SetScript("OnUpdate", XPerl_Player_OnUpdate)
	self:SetScript("OnEvent", XPerl_Player_OnEvent)

	XPerl_Player_InitDK(self)
	XPerl_Player_InitClassResource(self)

	local showmenu = function()
		ToggleDropDownMenu(1, nil, PlayerFrameDropDown, "PlayerFrame", 106, 27)
	end
	XPerl_SecureUnitButton_OnLoad(self, "player", nil, PlayerFrameDropDown, XPerl_ShowGenericMenu)	--PlayerFrame.menu)
	XPerl_SecureUnitButton_OnLoad(self.nameFrame, "player", nil, PlayerFrameDropDown, XPerl_ShowGenericMenu) --PlayerFrame.menu)

	if (XPerlDB) then
		self.conf = XPerlDB.player
	end
	XPerl_RegisterOptionChanger(XPerl_Player_Set_Bits, self)

	XPerl_RegisterHighlight(self.highlight, 3)
	XPerl_Highlight:Register(XPerl_Player_HighlightCallback, self)
	XPerl_RegisterPerlFrames(self, {self.nameFrame, self.statsFrame, self.levelFrame, self.portraitFrame, self.groupFrame, self.runes})

	self.FlashFrames = {self.portraitFrame, self.nameFrame, self.levelFrame, self.statsFrame, self.runes}

	XPerl_Player_OnLoad = nil
end

-- XPerl_Player_HighlightCallback(updateName)
function XPerl_Player_HighlightCallback(self, updateGUID)
	if (updateGUID == UnitGUID(self.unit)) then
		XPerl_Highlight:SetHighlight(self, updateGUID)
	end
end
local function XPerl_Player_CombatFlash(self, elapsed, argNew, argGreen)
	if (XPerl_CombatFlashSet(self, elapsed, argNew, argGreen)) then
		XPerl_CombatFlashSetFrames(self)
	end
end
-- XPerl_Player_SetGray
local function XPerl_Player_SetGray(self)
	XPerl_Unit_SetGrey(self, self.statsFrame.healthBar)	
	XPerl_Unit_HealthBar_Update(self.statsFrame.healthBar, self.unit)
	XPerl_Unit_SetGrey(self, self.statsFrame.manaBar)
	XPerl_Unit_ManaBar_Update(self.statsFrame.manaBar, self.unit)
	XPerl_Unit_SetGrey(self, self.statsFrame.druidBar)
	XPerl_Unit_ManaBar_Update(self.statsFrame.druidBar, self.unit)
end
-- CreateBar
local function CreateBar(self, name)
	local f = CreateFrame("StatusBar", self.statsFrame:GetName()..name, self.statsFrame, "XPerlStatusBar")
	f:SetPoint("TOPLEFT", self.statsFrame.manaBar, "BOTTOMLEFT", 0, 0)
	f:SetHeight(10)
	self.statsFrame[name] = f
	f:SetWidth(112)
	return f
end
-- MakeXPBars
local function MakeXPBars(self)
	local f = CreateBar(self, "xpBar")
	local f2 = CreateBar(self, "xpRestBar")
	f2:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
	f2:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)
	MakeXPBars = nil
end
-- MakeDruidBar
local function MakeDruidBar(self)
	local f = CreateBar(self, "druidBar")
	XPerl_Unit_ManaBar_Initialize (self, "player", f, false)
	XPerl_Unit_ManaBar_Update(f, "player")
	MakeDruidBar = nil
end
-- XPerl_Player_DruidBarUpdate
local function XPerl_Player_DruidBarUpdate(self)
	if (playerClass ~= "DRUID") then return end
	local druidBar, druidBarExtra = self.statsFrame.druidBar
	if (pconf.noDruidBar) then
		if (druidBar) then
			druidBar:Hide()
			XPerl_StatsFrameSetup(self, {self.statsFrame.xpBar, self.statsFrame.repBar})
			if (XPerl_Player_Buffs_Position) then
				XPerl_Player_Buffs_Position(self)
			end
		end
		return
	elseif (not druidBar) and (MakeDruidBar) then
		MakeDruidBar(self)
		druidBar = self.statsFrame.druidBar
	end

	if (UnitPowerType(self.unit) > 0) then
		if (pconf.percent) then
			druidBar.percent:Show()
		end
		druidBar.text:Show()		
		druidBar:Show()
		druidBar:SetHeight(10)
		druidBarExtra = 1
	else
		druidBar.percent:Hide()
		druidBar.text:Hide()
		druidBar:Hide()
		druidBar:SetHeight(1)
		druidBarExtra = 0
	end

	--if (InCombatLockdown()) then
	--	XPerl_ProtectedCall(XPerl_Player_DruidBarUpdate, self)
	--else
		local h = 40 + ((druidBarExtra + (pconf.repBar or 0) + (pconf.xpBar or 0)) * 10)
		if (pconf.extendPortrait) then
			self.portraitFrame:SetHeight(22 + h)
		end
		self.statsFrame:SetHeight(h)
	--end

	XPerl_StatsFrameSetup(self, {druidBar, self.statsFrame.xpBar, self.statsFrame.repBar})
	if (XPerl_Player_Buffs_Position) then
		XPerl_Player_Buffs_Position(self)
	end
end
-- RaidTargetUpdate
local function XPerl_Player_RaidIconUpdate(self)
	XPerl_Update_RaidIcon(self.nameFrame.raidIcon, self.unit)
end
-- XPerl_Player_UpdateLevel
local function XPerl_Player_UpdateLevel(self)
	self.levelFrame.text:SetText(UnitLevel(self.unit))
end
-- XPerl_Player_UpdateClass
local function XPerl_Player_UpdateClass(self)
	playerClass = select(2, UnitClass(self.unit))
	playerName = UnitName(self.unit)
	local l, r, t, b = XPerl_ClassPos(playerClass)
	self.classFrame.tex:SetTexCoord(l, r, t, b)

	if (pconf.classIcon) then
		self.classFrame:Show()
	else
		self.classFrame:Hide()
	end
end
-- XPerl_Player_UpdateCombat
local function XPerl_Player_UpdateCombat(self)
	local nf = self.nameFrame
	if (self.inCombat) then
		nf.text:SetTextColor(1,0,0)
		nf.combatIcon:SetTexCoord(0.5, 1.0, 0.0, 0.5)
		nf.combatIcon:Show()
	else
		XPerl_Unit_UpdateName(self)
		if (IsResting()) then
			nf.combatIcon:SetTexCoord(0, 0.5, 0.0, 0.5)
			nf.combatIcon:Show()
		else
			nf.combatIcon:Hide()
		end
	end
end
-- XPerl_Player_UpdateLeader()
local function XPerl_Player_UpdateLeader(self)
	-- Leader
	if (UnitIsGroupLeader("player")) then
		self.nameFrame.leaderIcon:Show()
	else
		self.nameFrame.leaderIcon:Hide()
	end
	-- Loot Master
	local lootMethod, lootMaster
	lootMethod, lootMaster = GetLootMethod()
	if (lootMaster == 0 and IsInGroup()) then
		self.nameFrame.masterIcon:Show()
	else
		self.nameFrame.masterIcon:Hide()
	end
	-- Group Number
	if (pconf and pconf.partyNumber and GetNumGroupMembers() > 0) then
		for i = 1,GetNumGroupMembers() do
			local name, rank, subgroup = GetRaidRosterInfo(i)
			if (UnitIsUnit("raid"..i, "player")) then
				if (pconf.withName) then
					self.nameFrame.group:SetFormattedText(XPERL_RAID_GROUPSHORT, subgroup)
					self.nameFrame.group:Show()
					self.groupFrame:Hide()
					return
				else
					self.groupFrame.text:SetFormattedText(XPERL_RAID_GROUP, subgroup)
					self.groupFrame:Show()
					self.nameFrame.group:Hide()
					return
				end
			end
		end
	end

	self.nameFrame.group:Hide()
	self.groupFrame:Hide()
end
-- XPerl_Player_UpdatePVP
local function XPerl_Player_UpdatePVP(self)
	local factionGroup = UnitFactionGroup("player")
	local nf = self.nameFrame
	
	if (pconf.pvpIcon and UnitIsPVPFreeForAll("player")) then
		if (not nf.pvp:IsShown()) then
			PlaySound("igPVPUpdate")
		end
		nf.pvp.icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-FFA")
		nf.pvp:Show()
		nf.pvp.PVPTimerText:Hide()
		nf.pvp.PVPTimerText.timeLeft = nil
	elseif (pconf.pvpIcon and factionGroup and factionGroup ~= "Neutral" and UnitIsPVP("player")) then
		if (not nf.pvp:IsShown()) then
			PlaySound("igPVPUpdate")
		end
		nf.pvp.icon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..factionGroup)
		nf.pvp:Show()
	else
		nf.pvp:Hide()
		nf.pvp.PVPTimerText.timeLeft = nil
	end
	
	if ( IsPVPTimerRunning() ) then
		nf.pvp.PVPTimerText:Show()
		nf.pvp.PVPTimerText.timeLeft = GetPVPTimer()
    else
		nf.pvp.PVPTimerText:Hide()
		nf.pvp.PVPTimerText.timeLeft = nil
    end
end
-- XPerl_Player_UpdateXP
local function XPerl_Player_UpdateXP(self)
	if (pconf.xpBar) then
		local xpBar = self.statsFrame.xpBar
		if (xpBar) then
			local restBar = self.statsFrame.xpRestBar
			local playerxp = UnitXP("player")
			local playerxpmax = UnitXPMax("player")
			local playerxprest = GetXPExhaustion() or 0
			xpBar:SetMinMaxValues(0, playerxpmax)
			restBar:SetMinMaxValues(0, playerxpmax)
			xpBar:SetValue(playerxp)

			local w = xpBar:GetRight() - xpBar:GetLeft()
			for mode = 1,3 do
				local suffix
				if (playerxprest > 0) then
					if (mode == 1) then
						suffix = format(" +%d", playerxprest)
					elseif (mode == 2) then
						if (playerxprest >= 1000000) then
							suffix = format(" +%.1fM", playerxprest / 1000000)
						else
							suffix = format(" +%.1fK", playerxprest / 1000)
						end
					else
						if (playerxprest >= 1000000) then
							suffix = format(" +%dM", playerxprest / 1000000)
						else
							suffix = format(" +%dK", playerxprest / 1000)
						end
					end

					color = {r = 0.3, g = 0.3, b = 1}
				else
					color = {r = 0.6, g = 0, b = 0.6}
				end

				if (pconf.xpDeficit) then
					XPerl_SetValuedText(xpBar.text, playerxp - playerxpmax, playerxpmax, suffix)
				else
					XPerl_SetValuedText(xpBar.text, playerxp, playerxpmax, suffix)
				end
				if (xpBar.text:GetStringWidth() <= w) then
					break
				end
			end

			xpBar:SetStatusBarColor(color.r, color.g, color.b, 1)
			xpBar.bg:SetVertexColor(color.r, color.g, color.b, 0.25)
			xpBar.tex:SetTexCoord(0, (playerxp * 100 / playerxpmax), 0, 1)

			restBar:SetValue(playerxp + playerxprest)
			restBar:SetStatusBarColor(color.r, color.g, color.b, 0.5)
			restBar.tex:SetTexCoord(0, (playerxp * 100 / playerxpmax), 0, 1)
			restBar.bg:SetVertexColor(color.r, color.g, color.b, 0.5)
			xpBar.percent:SetFormattedText(percD, (playerxp * 100) / playerxpmax)
		end
	end
end
-- XPerl_Player_UpdateRep
local function XPerl_Player_UpdateRep(self)
	if (pconf and pconf.repBar) then
		local rb = self.statsFrame.repBar
		if (rb) then
			local name, reaction, min, max, value = GetWatchedFactionInfo()
			local color

			if (name) then
				color = FACTION_BAR_COLORS[reaction]
			else
				name = XPERL_LOC_NONEWATCHED
				max = 1
				min = 0
				value = 0
				color = FACTION_BAR_COLORS[4]
			end

			value = value - min
			max = max - min
			min = 0

			rb:SetMinMaxValues(min, max)
			rb:SetValue(value)

			rb:SetStatusBarColor(color.r, color.g, color.b, 1)
			rb.bg:SetVertexColor(color.r, color.g, color.b, 0.25)

			local perc = (value * 100) / max
			rb.percent:SetFormattedText(perc1F, perc)

			rb.tex:SetTexCoord(0, perc/100, 0, 1)

			if (max == 1) then
				rb.text:SetText(name)
			else
				rb.text:SetFormattedText("%d/%d", value, max)
			end
		end
	end
end
-- XPerl_Player_UpdateBuffs
local function XPerl_Player_UpdateBuffs(self)
	XPerl_CheckDebuffs(self, self.unit)

	if (pconf.fullScreen.enable) then
		if (self.isOutOfControl and not UnitOnTaxi("player")) then
			XPerl_PlayerStatus_OnUpdate(self)
		end
	end
end
-- UpdateAssignedRoles
local function UpdateAssignedRoles(self)
	local unit = self.unit
	local icon = self.nameFrame.roleIcon
	--if (select(2, IsInInstance()) == "party") then
		role = UnitGroupRolesAssigned(unit)
		if role == "TANK" then
			icon:SetTexture("Interface\\GroupFrame\\UI-Group-MainTankIcon")
		elseif role == "HEALER" then
			icon:SetTexture("Interface\\Addons\\XPerl\\Images\\XPerl_RoleHealer_old")
		elseif role == "DAMAGER" then
			icon:SetTexture("Interface\\GroupFrame\\UI-Group-MainAssistIcon")
		end
	--end
	icon:SetTexCoord(0,1,0,1)
	if ( role == "TANK" or role == "HEALER" or role == "DAMAGER") then
		if not (conf and conf.xperlOldroleicons) then
			icon:SetTexture("Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES")
			icon:SetTexCoord(GetTexCoordsForRoleSmallCircle(role))
		end
		icon:Show()
	else
		icon:Hide()
	end
end

-- XPerl_Player_UpdateDisplay
function XPerl_Player_UpdateDisplay (self)
	if ( UnitExists("player") ) then
		XPerl_Player_DruidBarUpdate(self)
		XPerl_Player_UpdateLevel(self)
		XPerl_Player_UpdateClass(self)
		XPerl_Player_RaidIconUpdate(self)
		XPerl_Player_UpdateLeader(self)
		XPerl_Player_UpdateCombat(self)
		XPerl_Player_UpdatePVP(self)
		XPerl_Player_SetGray(self)
		XPerl_Player_UpdateBuffs(self)
		XPerl_Player_UpdateXP(self)
		XPerl_Player_UpdateRep(self)
		UpdateAssignedRoles(self)
	end
end

-------------------------
-- The Update Function --
-------------------------
function XPerl_Player_OnUpdate(self, elapsed)
	local PVPFrame = self.nameFrame.pvp.PVPTimerText
	if (PVPFrame.timeLeft) then
		PVPFrame.timeLeft = PVPFrame.timeLeft - elapsed*1000
		local timeLeft = PVPFrame.timeLeft
		if ( timeLeft < 0 ) then
			PVPFrame:Hide()
		else
			PVPFrame:Show()
		end
		PVPFrame:SetFormattedText(SecondsToTimeAbbrev(floor(timeLeft/1000)))
	else
		PVPFrame:Hide()
	end

	if (self.PlayerFlash) then
		XPerl_Player_CombatFlash(self, elapsed, false)
	end

	--[[if (IsResting() and UnitLevel("player") < 90) then
		self.restingDelay = (self.restingDelay or 2) - elapsed
		if (self.restingDelay <= 0) then
			self.restingDelay = 2
			XPerl_Player_UpdateXP(self)
		end
	end]]--

	CombatFeedback_OnUpdate(self, elapsed)
end

-- XPerl_PlayerStatus_OnUpdate
function XPerl_PlayerStatus_OnUpdate(self, val, max)
	if (pconf.fullScreen.enable) then
		local testLow = pconf.fullScreen.lowHP / 100
		local testHigh = pconf.fullScreen.highHP / 100

		if (val and max) then
			local test = val / max

			if ( test <= testLow and not XPerl_LowHealthFrame.frameFlash and not UnitIsDeadOrGhost("player")) then
				XPerl_FrameFlash(XPerl_LowHealthFrame)
			elseif ( (test >= testHigh and XPerl_LowHealthFrame.frameFlash) or UnitIsDeadOrGhost("player") ) then
				XPerl_FrameFlashStop(XPerl_LowHealthFrame, "out")
			end
			return
		else
			if (self.isOutOfControl and not XPerl_OutOfControlFrame.frameFlash) then
				XPerl_FrameFlash(XPerl_OutOfControlFrame)
			elseif (not self.isOutOfControl and XPerl_OutOfControlFrame.frameFlash) then
				XPerl_FrameFlashStop(XPerl_OutOfControlFrame, "out")
			end
			return
		end
	end

	if (XPerl_LowHealthFrame.frameFlash) then
		XPerl_FrameFlashStop(XPerl_LowHealthFrame)
	end
	if (XPerl_OutOfControlFrame.frameFlash) then
		XPerl_FrameFlashStop(XPerl_OutOfControlFrame)
	end
end

-------------------
-- Event Handler --
-------------------
function XPerl_Player_OnEvent(self, event, unitID, ...)
	--print (self:GetName().."-"..event.."-"..(unitID or "nil"))
	local func = XPerl_Player_Events[event]
	if (func) then
		if (strsub(event, 1, 5) == "UNIT_") then
			if (unitID == "player" or unitID == "vehicle") then
				XPerl_Player_Events[event](self, ...)
			end
		else
			XPerl_Player_Events[event](self, event, unitID, ...)
		end
	else
		XPerl_Unit_OnEvent(self, event, unitID)
	end
end

-- PLAYER_ENTERING_WORLD
function XPerl_Player_Events:PLAYER_ENTERING_WORLD()
	self.unit = UnitHasVehiclePlayerFrameUI("player") and "vehicle" or "player"
	self:SetAttribute("unit", self.unit)
	if (XPerl_ArcaneBar_SetUnit) then
		XPerl_ArcaneBar_SetUnit(self.nameFrame, self.unit)
	end
	XPerl_Player_UpdateDisplay(self)

	--fix runes on player load
	if self.runes then
		for i = 1,6 do
			if self.runes.list[i] then
				local rune = self.runes.list[i]
				RuneButton_Update(rune, rune:GetID(), true)
			end
		end
	end
end
-- UNIT_LEVEL
function XPerl_Player_Events:UNIT_LEVEL()
	XPerl_Player_UpdateLevel(self)
	XPerl_Player_UpdateXP(self)
end
-- RAID_TARGET_UPDATE
function XPerl_Player_Events:RAID_TARGET_UPDATE()
	XPerl_Player_RaidIconUpdate(XPerl_Player)
end
-- PLAYER_ROLES_ASSIGNED
function XPerl_Player_Events:PLAYER_ROLES_ASSIGNED()
	UpdateAssignedRoles(self)
end
-- UNIT_FLAGS
function XPerl_Player_Events:UNIT_FLAGS()
	XPerl_Player_SetGray(self)
end
-- UNIT_FACTION, PLAYER_FLAGS_CHANGED
function XPerl_Player_Events:UNIT_FACTION()
	XPerl_Player_UpdatePVP(self)
	XPerl_Player_UpdateCombat(self)
	XPerl_Player_SetGray(self)
end
XPerl_Player_Events.PLAYER_FLAGS_CHANGED	= XPerl_Player_Events.UNIT_FACTION
-- PLAYER_REGEN_DISABLED, PLAYER_REGEN_ENABLED
function XPerl_Player_Events:PLAYER_REGEN_DISABLED()
	self.inCombat = 1
	XPerl_Player_UpdateCombat(self)
end
function XPerl_Player_Events:PLAYER_REGEN_ENABLED()
	self.inCombat = nil
	XPerl_Player_UpdateCombat(self)
end
-- PLAYER_UPDATE_RESTING
function XPerl_Player_Events:PLAYER_UPDATE_RESTING()
	XPerl_Player_UpdateCombat(self)
end
-- UPDATE_EXHAUSTION
function XPerl_Player_Events:UPDATE_EXHAUSTION()
	XPerl_Player_UpdateXP(self)
end
-- PARTY_LOOT_METHOD_CHANGED, PARTY_LEADER_CHANGED, GROUP_ROSTER_UPDATE
function XPerl_Player_Events:PARTY_LOOT_METHOD_CHANGED()
	XPerl_Player_UpdateLeader(self)
end
XPerl_Player_Events.PARTY_LEADER_CHANGED	= XPerl_Player_Events.PARTY_LOOT_METHOD_CHANGED
XPerl_Player_Events.GROUP_ROSTER_UPDATE		= XPerl_Player_Events.PARTY_LOOT_METHOD_CHANGED
-- UNIT_ENTERED_VEHICLE, UNIT_EXITED_VEHICLE
function XPerl_Player_Events:UNIT_ENTERED_VEHICLE(showVehicle)
	if (showVehicle) then
		self.unit = "vehicle"
		if (XPerl_ArcaneBar_SetUnit) then
			XPerl_ArcaneBar_SetUnit(self.nameFrame, "vehicle")
		end
		if (not InCombatLockdown()) then
			self:SetAttribute("unit", "vehicle")
		end
		XPerl_Player_UpdateDisplay(self)
	end
end
function XPerl_Player_Events:UNIT_EXITED_VEHICLE()
	if (self.unit ~= "player") then
		self.unit = "player"
		if (XPerl_ArcaneBar_SetUnit) then
			XPerl_ArcaneBar_SetUnit(self.nameFrame, "player")
		end
		if (not InCombatLockdown()) then
			self:SetAttribute("unit", "player")
		end
		XPerl_Player_UpdateDisplay(self)
	end
end
-- UNIT_COMBAT
function XPerl_Player_Events:UNIT_COMBAT(...)
	local action, descriptor, damage, damageType = select(1, ...)
	
	if (pconf.hitIndicator and pconf.portrait) then
		if (action ~= "HEAL" or UnitHealth(self.unit) < UnitHealthMax(self.unit)) then
			CombatFeedback_OnCombatEvent(self, ...)
		end
	end

	if (action == "HEAL") then
		XPerl_Player_CombatFlash(self, elapsed, true, true)
	elseif (damage and damage > 0) then
		XPerl_Player_CombatFlash(self, elapsed, true)
	end
end

-- OTHER EVENTS RELATED ONLY TO PLAYER FRAME
-- UNIT_AURA
function XPerl_Player_Events:UNIT_AURA()
	XPerl_Player_UpdateBuffs(self)
end
-- PLAYER_XP_UPDATE
function XPerl_Player_Events:PLAYER_XP_UPDATE()
	XPerl_Player_UpdateXP(self)
end
-- UPDATE_FACTION
function XPerl_Player_Events:UPDATE_FACTION()
	XPerl_Player_UpdateRep(self)
end
-- PLAYER_CONTROL_LOST
function XPerl_Player_Events:PLAYER_CONTROL_LOST()
	if (pconf.fullScreen.enable and not UnitOnTaxi("player")) then
		self.isOutOfControl = 1
	end
end
-- PLAYER_CONTROL_GAINED
function XPerl_Player_Events:PLAYER_CONTROL_GAINED()
	self.isOutOfControl = nil
	if (pconf.fullScreen.enable) then
		XPerl_PlayerStatus_OnUpdate(self)
	end
end
-- PLAYER_TALENT_UPDATE, UPDATE_SHAPESHIFT_FORM
function XPerl_Player_Events:PLAYER_TALENT_UPDATE()
	XPerl_Player_ShowClassResource(self)
	XPerl_Player_DruidBarUpdate(self)
end
XPerl_Player_Events.UPDATE_SHAPESHIFT_FORM	= XPerl_Player_Events.PLAYER_TALENT_UPDATE

-- RUNE_POWER_UPDATE
function XPerl_Player_Events:RUNE_POWER_UPDATE(event, runeIndex, isEnergize)
	if runeIndex and runeIndex >= 1 and runeIndex <= 6 then
		local runeButton = _G['XPerl_RuneButtonIndividual' .. runeIndex]
		if (runeButton) then
			local cooldown = _G['XPerl_RuneButtonIndividual' .. runeIndex .. 'Cooldown']
			local start, duration, isReady = GetRuneCooldown(runeIndex)
			if not isReady then
				if start and cooldown then
					CooldownFrame_SetTimer(cooldown, start, duration, 1)
				end
				runeButton.energize:Stop()
			else
				if cooldown then
					cooldown:Hide()
				end
				runeButton.shine:SetVertexColor(1, 1, 1)
				RuneButton_ShineFadeIn(runeButton.shine)
			end
			if isEnergize then
				runeButton.energize:Play()
			end
		end
	else
		assert(false, 'Bad rune index')
	end
end
-- RUNE_TYPE_UPDATE
function XPerl_Player_Events:RUNE_TYPE_UPDATE(self, runeIndex )
	
	if runeIndex and runeIndex >= 1 and runeIndex <= 6 then
		
		RuneButton_Update(_G["XPerl_RuneButtonIndividual"..runeIndex], runeIndex)
	end
end

-- XPerl_Player_SetWidth
function XPerl_Player_SetWidth(self)

	pconf.size.width = max(0, pconf.size.width or 0)
	if (pconf.percent) then
		self.nameFrame:SetWidth(160 + pconf.size.width)
		self.statsFrame:SetWidth(160 + pconf.size.width)
		self.statsFrame.healthBar.percent:Show()
		self.statsFrame.manaBar.percent:Show()

		if (self.statsFrame.xpBar) then
			self.statsFrame.xpBar.percent:Show()
		end
		if (self.statsFrame.repBar) then
			self.statsFrame.repBar.percent:Show()
		end
	else
		self.nameFrame:SetWidth(128 + pconf.size.width)
		self.statsFrame:SetWidth(128 + pconf.size.width)
		self.statsFrame.healthBar.percent:Hide()
		self.statsFrame.manaBar.percent:Hide()
		if (self.statsFrame.xpBar) then
			self.statsFrame.xpBar.percent:Hide()
		end
		if (self.statsFrame.repBar) then
			self.statsFrame.repBar.percent:Hide()
		end
	end

	local h = 40 + ((((self.statsFrame.druidBar and self.statsFrame.druidBar:IsShown()) or 0) + (pconf.repBar or 0) + (pconf.xpBar or 0)) * 10)
	self.statsFrame:SetHeight(h)

	self:SetWidth((pconf.portrait or 0) * 62 + (pconf.percent or 0) * 32 + 124 + pconf.size.width)
	self:SetScale(pconf.scale)

	XPerl_StatsFrameSetup(self, {self.statsFrame.druidBar, self.statsFrame.xpBar, self.statsFrame.repBar})
	if (XPerl_Player_Buffs_Position) then
		XPerl_Player_Buffs_Position(self)
	end

	XPerl_Player_UpdateXP(self)

	XPerl_SavePosition(self, true)
	XPerl_RestorePosition(self)
end

-- XPerl_Player_SetTotems
function XPerl_Player_SetTotems(self, ...)
	if (pconf.totems and pconf.totems.enable) then
		TotemFrame:SetParent(XPerl_Player)
		TotemFrame:ClearAllPoints()
		TotemFrame:SetPoint("TOP", XPerl_Player, "BOTTOM", pconf.totems.offsetX, pconf.totems.offsetY)
		TotemFrame:SetScale(0.9)
	else
		TotemFrame:SetParent(PlayerFrame)
		TotemFrame:ClearAllPoints()
		TotemFrame:SetPoint("TOPLEFT", PlayerFrame, "BOTTOMLEFT", 99, 38)
	end
end

-- XPerl_Player_Set_Bits()

function XPerl_Player_Set_Bits(self)
	XPerl_Unit_Initialize(self, self.nameFrame.text, self.statsFrame.healthBar, self.statsFrame.manaBar, self.statsFrame.expectedHealth)
	if (XPerl_ArcaneBar_RegisterFrame and not self.nameFrame.castBar) then
		XPerl_ArcaneBar_RegisterFrame(self.nameFrame, UnitHasVehiclePlayerFrameUI("player") and "vehicle" or "player")
	end
	
	playerClass = select(2,UnitClass("player"))

	if (pconf.portrait) then
		self.portraitFrame:Show()
		self.portraitFrame:SetWidth(60)
	else
		self.portraitFrame:Hide()
		self.portraitFrame:SetWidth(3)
	end

	if (pconf.level) then
		self.levelFrame:Show()
	else
		self.levelFrame:Hide()
	end

	if (pconf.repBar) then
		if (not self.statsFrame.repBar) then
			CreateBar(self, "repBar")
		end

		self.statsFrame.repBar:Show()
	else
		if (self.statsFrame.repBar) then
			self.statsFrame.repBar:Hide()
		end
	end

	if (pconf.xpBar) then
		if (not self.statsFrame.xpBar) then
			MakeXPBars(self)
		end

		self.statsFrame.xpBar:Show()
		self.statsFrame.xpRestBar:Show()
		self:RegisterEvent("PLAYER_XP_UPDATE")
	else
		if (self.statsFrame.xpBar) then
			self.statsFrame.xpBar:Hide()
			self.statsFrame.xpRestBar:Hide()
		end
		self:UnregisterEvent("PLAYER_XP_UPDATE")
	end

	if (pconf.values) then
		self.statsFrame.healthBar.text:Show()
		self.statsFrame.manaBar.text:Show()
		if (self.statsFrame.xpBar) then
			self.statsFrame.xpBar.text:Show()
		end
		if (self.statsFrame.repBar) then
			self.statsFrame.repBar.text:Show()
		end
	else
		self.statsFrame.healthBar.text:Hide()
		self.statsFrame.manaBar.text:Hide()
		if (self.statsFrame.xpBar) then
			self.statsFrame.xpBar.text:Hide()
		end
		if (self.statsFrame.repBar) then
			self.statsFrame.repBar.text:Hide()
		end
	end

	XPerl_Player_SetWidth(self)

	local h1 = self.nameFrame:GetHeight() + self.statsFrame:GetHeight() - 2
	local h2 = self.portraitFrame:GetHeight()
	XPerl_SwitchAnchor(self, "TOPLEFT")
	self:SetHeight(max(h1, h2))

	self.portraitFrame:SetHeight(62 + ((pconf.extendPortrait or 0)*((pconf.xpBar or 0) + (pconf.repBar or 0)) * 10))

	self.highlight:ClearAllPoints()
	if (not pconf.level and not pconf.classIcon and (not XPerlConfigHelper or XPerlConfigHelper.ShowTargetCounters == 0)) then
		self.highlight:SetPoint("TOPLEFT", self.portraitFrame, "TOPLEFT", 0, 0)
	else
		self.highlight:SetPoint("TOPLEFT", self.levelFrame, "TOPLEFT", 0, 0)
	end
	self.highlight:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0)

	if (playerClass == "SHAMAN" or playerClass == "DRUID" or playerClass == "MONK" or playerClass == "PRIEST") then
		if (not pconf.totems) then
			pconf.totems = {
				enable = true,
				offsetX = 0,
				offsetY = 0
			}
		end

		if (not self.totemHooked) then
			self.totemHooked = true
			hooksecurefunc("TotemFrame_Update", XPerl_Player_SetTotems)
		end
	end
	
	self:SetAlpha(conf.transparency.frame)

	self.buffOptMix = nil	
	if (XPerl_Player_BuffSetup) then
		if (self.buffFrame) then
			self.buffOptMix = nil
			XPerl_Player_BuffSetup(XPerl_Player)
		end
	end

	if (XPerl_Voice) then
		XPerl_Voice:Register(self)
	end	
	
	XPerl_Player_SetupDK(self)
	XPerl_Player_SetupClassResource(self)
	XPerl_Player_UpdateDisplay(self)
end

--XPerl_Player_ShowClassResource
function XPerl_Player_ShowClassResource(self)
	if (select(2,UnitClass("player")) == "DRUID") then
		local form = GetShapeshiftFormID()
		if ((form and form ~= MOONKIN_FORM) or GetSpecialization() ~= 1) then
			self.classResource:Hide()
		else
			self.classResource:Show()
		end
	else
		self.classResource:Show()
	end
end

--XPerl_Player_MoveClassResource
function XPerl_Player_MoveClassResource(self, mtype)
	if mtype == 1 then	
		if (not pconf.dockRunes) then
			self:StartMoving()
		end
	else
		self:StopMovingOrSizing()
		XPerl_SavePosition(self)
	end
end

--XPerl_Player_InitClassResource
function XPerl_Player_InitClassResource(self)
	local class = select(2,UnitClass("player"))
	local classResourceFrame = nil
	local x, y = 0, 0
	if ( class == "MONK" )  then
		classResourceFrame = MonkHarmonyBar
		x, y = 0, 4
	elseif ( class == "PALADIN") then
		classResourceFrame = PaladinPowerBar		
		x, y = -2, 3
	elseif ( class == "PRIEST") then
		classResourceFrame = PriestBarFrame
		x, y = 0, -8
	elseif ( class == "WARLOCK" )  then
		classResourceFrame = WarlockPowerFrame
		x, y = -15, 4
	elseif ( class == "DRUID") then
		classResourceFrame = EclipseBarFrame
		x, y = -2, 2
	else return end
	if classResourceFrame then
		classResourceFrame:SetMovable(true)
		classResourceFrame:RegisterForDrag("LeftButton")
		classResourceFrame:SetScript("OnDragStart", function(self)
				if (XPerlLocked == 0) then
					XPerl_Player_MoveClassResource(self:GetParent(), 1)
				end
			end)
		classResourceFrame:SetScript("OnDragStop", function(self)
				XPerl_Player_MoveClassResource(self:GetParent(), 0)
			end)	
		self.classResource.unit = classResourceFrame:GetParent().unit
		classResourceFrame:SetParent(self.classResource)
		classResourceFrame:ClearAllPoints()
		classResourceFrame:SetPoint("CENTER", x, y)
		classResourceFrame = nil
	end
	XPerl_Player_InitClassResource = nil
end
--XPerl_Player_SetupClassResource
function XPerl_Player_SetupClassResource(self)
	if (self.classResource) then
		if (not pconf or not pconf.showRunes) then
			self.classResource:Hide()
		else
			self.classResource:ClearAllPoints()
			if (not pconf or pconf.dockRunes) then
				self.classResource:SetPoint("TOPLEFT", self.statsFrame, "BOTTOMLEFT", 0, 0)
				self.classResource:SetPoint("BOTTOMRIGHT", self.statsFrame, "BOTTOMRIGHT", 0, -30) 
			else
				self.classResource:SetPoint("TOPLEFT", self.portraitFrame, "BOTTOMLEFT", 0, 2)
				XPerl_RestorePosition(self.classResource)
			end
			XPerl_Player_ShowClassResource(self)
		end
	end
end

-- XPerl_Player_InitDK
function XPerl_Player_InitDK(self)
	if (select(2, UnitClass("player")) == "DEATHKNIGHT") then
		if (not RuneFrame or RuneFrame:GetParent() ~= UIParent or not RuneFrame:IsShown()) then
			-- Only hijack runes if not already done so by another mod
			return
		end

		local p, f, r, x, y = RuneFrame:GetPoint(1)
		if (f ~= PlayerFrame or p ~= "TOP" or r ~= "BOTTOM") then
			-- More safety checking before we fudge it
			return
		end

		self.runes = CreateFrame("Frame", "XPerl_Runes", self)
		self.runes:SetPoint("TOPLEFT", self.statsFrame, "BOTTOMLEFT",  -(self.portraitFrame:GetWidth()-3), 2)
		self.runes:SetPoint("BOTTOMRIGHT", self.statsFrame, "BOTTOMRIGHT", 0, -30)

		self.runes:SetMovable(true)
		self.runes:RegisterForDrag("LeftButton")
		self.runes:SetScript("OnDragStart",
			function(self)
				if (not pconf.dockRunes) then
					self:StartMoving()
				end
			end)
		self.runes:SetScript("OnDragStop",
			function(self)
				if (not pconf.dockRunes) then
					self:StopMovingOrSizing()
					XPerl_SavePosition(self)
				end
			end)

		local bgDef = {bgFile = "Interface\\Addons\\XPerl\\Images\\XPerl_FrameBack",
				edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
				tile = true, tileSize = 32, edgeSize = 12,
				insets = { left = 3, right = 3, top = 3, bottom = 3 }
			}
		self.runes:SetBackdrop(bgDef)
		self.runes:SetBackdropColor(0, 0, 0, 0.8)
		self.runes:SetBackdropBorderColor(1, 1, 1, 1)
		self.runes.list = {}

		--Let's not do anything drastic to the original runeframe, let's just hide it
		--RuneFrame:SetParent(self.runes)
		RuneFrame:ClearAllPoints()
		RuneFrame:SetPoint("TOPLEFT", 5, -5)
		RuneFrame:SetPoint("BOTTOMRIGHT", -5, 5)

		RuneButtonIndividual1:Hide()
		RuneButtonIndividual2:Hide()
		RuneButtonIndividual3:Hide()
		RuneButtonIndividual4:Hide()
		RuneButtonIndividual5:Hide()
		RuneButtonIndividual6:Hide()

		local prev
		local idSwitch = {1, 2, 5, 6, 3, 4}	-- Non sequential rune IDs.. gg
		for i = 1,6 do
			local rune = CreateFrame("Button", "XPerl_RuneButtonIndividual"..i, self.runes, "XPerl_RuneButtonIndividualTemplate")
			self.runes.list[ idSwitch[i] ] = rune
			rune:EnableMouse(false)

			-- god only knows why blizzard did this...
			rune:SetID(i)

			if (i > 1) then
				rune:SetPoint("LEFT", prev, "RIGHT", 4 + ((i % 2) * 16), 0)
			else
				rune:SetPoint("LEFT", 10, 0)
			end
		
			prev = rune
		end

	end
	
	XPerl_Player_InitDK = nil
end
-- XPerl_Player_SetupDK
function XPerl_Player_SetupDK(self)
	if (self.runes) then
		if (not pconf or pconf.showRunes) then
			self.runes:Show()

			self.runes:ClearAllPoints()

			if (not pconf or pconf.dockRunes) then
				self.runes:SetPoint("TOPLEFT", self.statsFrame, "BOTTOMLEFT", -(self.portraitFrame:GetWidth()-3), 2)
				self.runes:SetPoint("BOTTOMRIGHT", self.statsFrame, "BOTTOMRIGHT", 0, -30)
			else
				self.runes:SetPoint("TOPLEFT", self.portraitFrame, "BOTTOMLEFT", 0, 2)
				self.runes:SetWidth(214)
				self.runes:SetHeight(32)
				XPerl_RestorePosition(self.runes)
			end

			for i = 1,6 do
				local rune = self.runes.list[i]
				rune:ClearAllPoints()
				rune:SetWidth(22)
				rune:SetHeight(22)

				if (self.runes:GetWidth() >= 200) then	-- pconf.portrait) then
				local dist = self.runes:GetWidth() / 3 
					if (i > 1) then
						rune:SetPoint("LEFT", prev, "RIGHT", 4 + ((i % 2) * dist - (i % 2) * 48), 0)
					else
						rune:SetPoint("LEFT", (self.runes:GetWidth() / 4 )-48, 0)
					end
				else
					if (i > 1) then
						rune:SetPoint("LEFT", prev, "RIGHT", 1 + ((i % 2) * 5), 0)
					else
						rune:SetPoint("LEFT", 6, 0)
					end
				end
				RuneButton_Update(rune, rune:GetID(), true)
				prev = rune
			end

		else
			self.runes:Hide()
		end
	end
end

